package cc.toph.progression.item.tier;

import cc.toph.progression.registry.ConfigRegistry;
import cc.toph.progression.registry.ConfigRegistry.TIER_ATTRIBUTES;
import cc.toph.progression.tag.Tags;
import net.minecraft.core.registries.BuiltInRegistries;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.sounds.SoundEvent;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.tags.TagKey;
import net.minecraft.world.item.ArmorItem;
import net.minecraft.world.item.ArmorMaterial;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.crafting.Ingredient;
import org.jetbrains.annotations.NotNull;

public enum ArmorTier implements Material<ArmorTier>, ArmorMaterial {
    COPPER(
            DurabilityRecord.fromMultiplier(10),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_GENERIC,
            Tags.ITEMS.COPPER_INGOT.tag(),
            0f,
            0f,
            ConfigRegistry.TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    EXPOSED_COPPER(
            DurabilityRecord.fromMultiplier(14),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_GENERIC,
            Tags.ITEMS.EXPOSED_COPPER_INGOT.tag(),
            0f,
            0f,
            ConfigRegistry.TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    WEATHERED_COPPER(
            DurabilityRecord.fromMultiplier(18),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_GENERIC,
            Tags.ITEMS.WEATHERED_COPPER_INGOT.tag(),
            0f,
            0f,
            ConfigRegistry.TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    OXIDIZED_COPPER(
            DurabilityRecord.fromMultiplier(20),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_GENERIC,
            Tags.ITEMS.OXIDIZED_COPPER_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    ROSE_GOLD(
            DurabilityRecord.fromMultiplier(15),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_GOLD,
            Tags.ITEMS.ROSE_GOLD_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    STURDY(
            DurabilityRecord.fromMultiplier(39),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_NETHERITE,
            Tags.ITEMS.STURDY_INGOT.tag(),
            0f,
            0f,
            ConfigRegistry.TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    STEEL(
            DurabilityRecord.fromMultiplier(33),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_IRON,
            Tags.ITEMS.STEEL_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    REFINED(
            DurabilityRecord.fromMultiplier(42),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_DIAMOND,
            Tags.ITEMS.REFINED_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    ECHO(
            DurabilityRecord.fromMultiplier(42),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_DIAMOND,
            Tags.ITEMS.ECHO_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    WHITHER(
            DurabilityRecord.fromMultiplier(42),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_DIAMOND,
            Tags.ITEMS.WHITHER_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    ),
    VOID(
            DurabilityRecord.fromMultiplier(42),
            new DefenseRecord(2, 3, 4, 2),
            25,
            SoundEvents.ARMOR_EQUIP_DIAMOND,
            Tags.ITEMS.VOID_INGOT.tag(),
            0f,
            0f,
            TIER_ATTRIBUTES.COPPER_ENABLED
    );

    private final DurabilityRecord durabilityRecord;
    private final DefenseRecord defenseRecord;
    private final int enchantmentValue;
    private final SoundEvent getEquipSound;
    private final TagKey<Item> repairIngredient;
    private final float toughness;
    private final float knockbackResistance;
    private final boolean enabled;
    // TODO: Armor Full Effect
    // TODO: Armor Ability

    ArmorTier(
            DurabilityRecord durabilityRecord,
            DefenseRecord defenseRecord,
            int enchantmentValue,
            SoundEvent getEquipSound,
            TagKey<Item> repairIngredient,
            float toughness,
            float knockbackResistance,
            boolean enabled
    ) {
        this.durabilityRecord = durabilityRecord;
        this.defenseRecord = defenseRecord;
        this.enchantmentValue = enchantmentValue;
        this.getEquipSound = getEquipSound;
        this.repairIngredient = repairIngredient;
        this.toughness = toughness;
        this.knockbackResistance = knockbackResistance;
        this.enabled = enabled;
        // TODO: Armor Full Effect
        // TODO: Armor Ability
    }

    @Override
    public int getDurabilityForType(ArmorItem.Type type) {
        return switch (type.getName()) {
            case "helmet" -> durabilityRecord.helmet();
            case "chestplate" -> durabilityRecord.chestplate();
            case "leggings" -> durabilityRecord.leggings();
            case "boots" -> durabilityRecord.boots();
            default -> 0;
        };
    }

    @Override
    public int getDefenseForType(ArmorItem.Type type) {
        return switch (type.getName()) {
            case "helmet" -> defenseRecord.helmet();
            case "chestplate" -> defenseRecord.chestplate();
            case "leggings" -> defenseRecord.leggings();
            case "boots" -> defenseRecord.boots();
            default -> 0;
        };
    }

    @Override
    public int getEnchantmentValue() {
        return enchantmentValue;
    }

    @Override
    public @NotNull SoundEvent getEquipSound() {
        return getEquipSound;
    }

    @Override
    public @NotNull Ingredient getRepairIngredient() {
        return Ingredient.of(repairIngredient);
    }

    @Override
    public @NotNull String getName() {
        return this.name().toLowerCase();
    }

    @Override
    public float getToughness() {
        return toughness;
    }

    @Override
    public float getKnockbackResistance() {
        return knockbackResistance;
    }

    @Override
    public ResourceLocation getIngredientLocation() {
        return repairIngredient.location();
    }

    @Override
    public TagKey<Item> getIngredientTag() {
        return repairIngredient;
    }

    @Override
    public Item getIngredientAsItem() {
        return BuiltInRegistries.ITEM.get(repairIngredient.location());
    }

    @Override
    public boolean isEnabled() {
        return enabled;
    }

    @Override
    public ArmorTier[] getValues() {
        return values();
    }

    protected record DurabilityRecord(
            int boots,
            int leggings,
            int chestplate,
            int helmet) {

        public static DurabilityRecord fromMultiplier(int multiplier) {
            return new DurabilityRecord(
                    13 * multiplier,
                    15 * multiplier,
                    16 * multiplier,
                    11 * multiplier
            );
        }
    }

    protected record DefenseRecord(
            int boots,
            int leggings,
            int chestplate,
            int helmet) {
    }

}
