package cc.toph.progression.item.weapon.compat;

import cc.toph.progression.item.ItemSpec;
import cc.toph.progression.item.tier.ToolTier;
import net.minecraft.resources.ResourceLocation;
import net.sweenus.simplyswords.item.SimplySwordsSwordItem;
import org.jetbrains.annotations.NotNull;

/**
 * Extended Class from Simply Swords to store sword type.
 * Storing Sword type will let us avoid all the static mess in registries.
 * This class stores all the data we need from the sword, so we can just
 * retrieve the object from the registries cast it and do whatever we need.
 */
public class SSSwordItem extends SimplySwordsSwordItem implements ItemSpec<SSSwordItem> {

    /**
     * The type of the weapon.
     */
    private final SSWeaponType TYPE;

    /**
     * Constructor for SimplyCompatSwordItem.
     *
     * @param toolMaterial The material of the tool, with some of its properties/stats.
     * @param type         The type of the weapon, which affects its modifiers.
     */
    public SSSwordItem(ToolTier material, SSWeaponType type) {
        super(
                material,
                (int) (material.getDamageModifier() + type.getPositiveModifier() - type.getNegativeModifier()),
                type.getAttackSpeed(),
                material.getRepairIngredient().toString()
        );

        this.TYPE = type;
    }

    public SSWeaponType getType() {
        return TYPE;
    }

    @Override
    public @NotNull ToolTier getTier() {
        return (ToolTier) super.getTier();
    }

    @Override
    public ResourceLocation getLocation() {
        return this.arch$registryName();
    }

    @Override
    public SSSwordItem getItemInstance() {
        return this;
    }
}