package cc.toph.progression.registry;

import cc.toph.progression.item.armor.Armor;
import cc.toph.progression.item.armor.UltimateArmor;
import cc.toph.progression.item.ingot.Ingot;
import cc.toph.progression.item.ingot.Spawnable;
import cc.toph.progression.item.tier.ArmorTier;
import cc.toph.progression.item.tier.ToolTier;
import cc.toph.progression.item.tool.Axe;
import cc.toph.progression.item.tool.Hoe;
import cc.toph.progression.item.tool.PickAxe;
import cc.toph.progression.item.tool.Shovel;
import cc.toph.progression.item.weapon.Sword;
import cc.toph.progression.item.weapon.compat.SSSwordItem;
import cc.toph.progression.item.weapon.compat.SSWeaponType;
import dev.architectury.registry.registries.DeferredRegister;
import net.minecraft.core.registries.Registries;
import net.minecraft.world.item.ArmorItem;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.SwordItem;
import net.sweenus.simplyswords.item.SimplySwordsSwordItem;

import java.util.ArrayList;
import java.util.List;

import static cc.toph.progression.util.Constants.MOD_ID;

public final class ItemsRegistry {
    public static void registerItems() {
        Ingots.register();
        Armors.register();
        Tools.register();
        Swords.register();
        // SimplySwords.register();

        // CompatRegistry.registerCompats();
    }

    public static final class Ingots {
        public static final DeferredRegister<Item> REGISTRY = DeferredRegister.create(MOD_ID, Registries.ITEM);

        /*
         Register ingots for each tool tier,
         although tiers are technically not ingots
         they have the same names so ¯\_(ツ)_/¯
        */
        private static void registerIngots() {
            for (ToolTier tier : ToolTier.values()) {
                if (tier.isEnabled()) {
                    switch (tier.getName()) {
                        case "REFINED":
                        case "ECHO":
                        case "WHITHER":
                        case "VOID":
                            REGISTRY.register(tier.getName() + "_ingot", () -> new Spawnable(new Item.Properties().arch$tab(TabRegistry.PROGRESSION)));
                        default:
                            REGISTRY.register(tier.getName() + "_ingot", () -> new Ingot(new Item.Properties().arch$tab(TabRegistry.PROGRESSION)));
                    }
                }
            }
        }

        public static List<Ingot> getIngots() {
            var ingots = new ArrayList<Ingot>();
            REGISTRY.forEach((item) -> {
                if (item.get() instanceof Ingot) {
                    ingots.add((Ingot) item.get());
                }
            });

            return ingots;
        }

        //TODO: haven't created them yet
        private static void registerCompounds() {
            //
        }

        private static void register() {
            registerIngots();
            // registerCompounds();
            REGISTRY.register();
        }

    }

    public static final class Armors {
        public static final DeferredRegister<Item> REGISTRY = DeferredRegister.create(MOD_ID, Registries.ITEM);

        public static List<Armor> getArmors() {
            var armors = new ArrayList<Armor>();
            REGISTRY.forEach((item) -> {
                if (item.get() instanceof Armor) {
                    armors.add((Armor) item.get());
                }
            });

            return armors;
        }

        private static void registerArmors() {
            for (ArmorTier tier : ArmorTier.values()) {
                for (ArmorItem.Type type : ArmorItem.Type.values()) {
                    if (tier.isEnabled()) {
                        switch (tier.getName()) {
                            case "REFINED":
                            case "ECHO":
                            case "WHITHER":
                            case "VOID":
                                REGISTRY.register(
                                        tier.getName() + "_" + type.getName(),
                                        () -> new UltimateArmor(tier, type, new Item.Properties().arch$tab(TabRegistry.PROGRESSION), null)
                                );
                            default:
                                REGISTRY.register(
                                        tier.getName() + "_" + type.getName(),
                                        () -> new Armor(tier, type, new Item.Properties().arch$tab(TabRegistry.PROGRESSION))
                                );
                        }
                    }
                }
            }
        }

        private static void register() {
            registerArmors();
            REGISTRY.register();
        }
    }

    public static final class Tools {
        public static final DeferredRegister<Item> AXE = DeferredRegister.create(MOD_ID, Registries.ITEM);
        public static final DeferredRegister<Item> PICKAXE = DeferredRegister.create(MOD_ID, Registries.ITEM);
        public static final DeferredRegister<Item> SHOVEL = DeferredRegister.create(MOD_ID, Registries.ITEM);
        public static final DeferredRegister<Item> HOE = DeferredRegister.create(MOD_ID, Registries.ITEM);

        private static void registerTools() {
            for (ToolTier tier : ToolTier.values()) {
                if (tier.isEnabled()) {
                    AXE.register(tier.getName() + "_axe", () -> new Axe(tier, new Item.Properties().arch$tab(TabRegistry.PROGRESSION)));
                    PICKAXE.register(tier.getName() + "_pickaxe", () -> new PickAxe(tier, new Item.Properties().arch$tab(TabRegistry.PROGRESSION)));
                    SHOVEL.register(tier.getName() + "_shovel", () -> new Shovel(tier, new Item.Properties().arch$tab(TabRegistry.PROGRESSION)));
                    HOE.register(tier.getName() + "_hoe", () -> new Hoe(tier, new Item.Properties().arch$tab(TabRegistry.PROGRESSION)));
                }
            }
        }

        public static List<Axe> getAxes() {
            var axes = new ArrayList<Axe>();
            AXE.forEach((item) -> {
                if (item.get() instanceof Axe) {
                    axes.add((Axe) item.get());
                }
            });
            return axes;
        }

        public static List<PickAxe> getPickAxes() {
            var pickAxes = new ArrayList<PickAxe>();
            PICKAXE.forEach((item) -> {
                if (item.get() instanceof PickAxe) {
                    pickAxes.add((PickAxe) item.get());
                }
            });
            return pickAxes;
        }

        public static List<Shovel> getShovels() {
            var shovels = new ArrayList<Shovel>();
            SHOVEL.forEach((item) -> {
                if (item.get() instanceof Shovel) {
                    shovels.add((Shovel) item.get());
                }
            });
            return shovels;
        }

        public static List<Hoe> getHoes() {
            var hoes = new ArrayList<Hoe>();
            HOE.forEach((item) -> {
                if (item.get() instanceof Hoe) {
                    hoes.add((Hoe) item.get());
                }
            });
            return hoes;
        }

        private static void register() {
            registerTools();
            AXE.register();
            PICKAXE.register();
            SHOVEL.register();
            HOE.register();
        }
    }

    public static final class Swords {
        public static final DeferredRegister<Item> REGISTRY = DeferredRegister.create(MOD_ID, Registries.ITEM);

        private static void registerSwords() {
            for (ToolTier tier : ToolTier.values()) {
                if (tier.isEnabled()) {
                    REGISTRY.register(tier.getName() + "_sword"
                            , () -> new Sword(tier, new Item.Properties().arch$tab(TabRegistry.PROGRESSION))
                    );
                }
            }
        }

        public static List<Sword> getSwords() {
            var swords = new ArrayList<Sword>();
            REGISTRY.forEach((item) -> {
                if (item.get() instanceof Sword) {
                    swords.add((Sword) item.get());
                }
            });

            return swords;
        }

        private static void register() {
            registerSwords();
            REGISTRY.register();
        }
    }

    public static final class SimplySwords {
        public static final DeferredRegister<Item> REGISTRY = DeferredRegister.create(MOD_ID, Registries.ITEM);

        // FIXME: pasted old code, needs to be updated probably
        private static void registerSSwords() {
            for (SSWeaponType type : SSWeaponType.values()) {
                for (ToolTier material : ToolTier.values()) {
                    if (material.isEnabled()) {
                        var name = material.getName() + "_" + type.getWeaponNameSuffix();
                        REGISTRY.register(name, () -> new SSSwordItem(material, type));
                    }
                }
            }
        }

        public static List<SimplySwordsSwordItem> getSwords() {
            var swords = new ArrayList<SimplySwordsSwordItem>();
            REGISTRY.forEach((item) -> {
                if (item.get() instanceof SwordItem) {
                    swords.add((SimplySwordsSwordItem) item.get());
                }
            });

            return swords;
        }

        private static void register() {

            //TODO: if SS enabled register SS swords
            //  for (ToolTier tier : ToolTier.values()) {
            //      if (tier.isEnabled()) {
            //          REGISTRY.register(tier.getName() + "_sword", () -> new Weapon(new Item.Properties()));
            //      }
            //  }

            registerSSwords();
            REGISTRY.register();
        }
    }
}